/***
 * Think ADL Parser
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: think@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.idl.remote;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.AbstractInterfaceLoader;
import org.objectweb.fractal.cecilia.adl.idl.ast.Type;
import org.objectweb.fractal.cecilia.adl.idl.remote.ast.RemoteInterfaceDefinition;

/**
 * Delegating {@link org.objectweb.fractal.adl.interfaces.IDLLoader} that checks
 * that interface declared <code>remote</code> are correct. This is an
 * abstract class, sub classes must implements the
 * {@link #isSerializable(Type) method}.
 */
public abstract class AbstractRemoteIDLLoader extends AbstractInterfaceLoader {

  public Object loadInterface(final String signature,
      final Map<Object, Object> context) throws ADLException {
    final Object itf = clientLoader.loadInterface(signature, context);
    if (itf instanceof RemoteInterfaceDefinition
        && RemoteInterfaceDefinition.TRUE
            .equals(((RemoteInterfaceDefinition) itf).getRemote())) {
      checkRemoteInterface((RemoteInterfaceDefinition) itf, context);
    }
    return itf;
  }

  protected abstract void checkRemoteInterface(
      final RemoteInterfaceDefinition definition,
      final Map<Object, Object> context) throws ADLException;

  protected abstract boolean isSerializable(final Type type);

  // TODO remove this method when the bug is fixed in Plugin Manager
  @Override
  public void bindFc(final String s, final Object o) {
    if ("node-factory".equals(s)) {
      return;
    }
    super.bindFc(s, o);
  }
}
