
package org.objectweb.fractal.cecilia.primitive.cbst.skelmem;

import org.objectweb.fractal.cecilia.adl.file.CodeWriter;
import org.objectweb.fractal.cecilia.adl.idl.ast.ArrayOf;
import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.InterfaceDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.Method;
import org.objectweb.fractal.cecilia.adl.idl.ast.Parameter;
import org.objectweb.fractal.cecilia.adl.idl.ast.ParameterContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.PointerOf;
import org.objectweb.fractal.cecilia.adl.idl.ast.PrimitiveType;
import org.objectweb.fractal.cecilia.adl.idl.ast.PrimitiveType.PrimitiveTypeEnum;
import org.objectweb.fractal.cecilia.adl.idl.ast.Type;
import org.objectweb.fractal.cecilia.adl.interfaces.AbstractInterfaceAdapterImplementationVisitor;

import static org.objectweb.fractal.cecilia.adl.idl.util.CUtil.buildComplexOrPrimitiveType;
import static org.objectweb.fractal.cecilia.adl.idl.util.CUtil.buildMethodDefinition;
import static org.objectweb.fractal.cecilia.adl.idl.util.Util.getContainedType;
import static org.objectweb.fractal.cecilia.adl.idl.util.Util.getPrimitiveType;

public class ImplementationVisitor
    extends
      AbstractInterfaceAdapterImplementationVisitor {

  @Override
  protected AbstractInterfaceAdapterImplmentationTask createImplementationTask(
      final InterfaceDefinition itf) {
    return new ImplementationTask(itf);
  }

  public static class ImplementationTask
      extends
        AbstractInterfaceAdapterImplmentationTask {

    private final CodeWriter methCode_cw = new CodeWriter();

    private static enum ParseArrayMode {
      TYPE, SIZEOF
    };

    private static String buildSizeableArray(Type type,
        final ParseArrayMode What) {
      final StringBuffer result = new StringBuffer();
      String arraySpec = "";

      while (type instanceof ArrayOf) {
        final String size = ((ArrayOf) type).getSize();
        if (size != null) {
          arraySpec = "[" + size + "]" + arraySpec;
        } else {
          arraySpec = "[]" + arraySpec;
        }
        type = getContainedType((ArrayOf) type);
      }
      if (What == ParseArrayMode.TYPE)
        result.append(buildComplexOrPrimitiveType(type));
      if (What == ParseArrayMode.SIZEOF) result.append(arraySpec);
      return result.toString();
    }

    public ImplementationTask(final InterfaceDefinition itf) {
      super(itf);
    }

    public void enterInterface(final InterfaceDefinition itf) throws Exception {
      cw.appendln("DECLARE_DATA {};");
      cw.appendln("#include <cecilia.h>");
    }

    public void leaveInterface(final InterfaceDefinition itf) throws Exception {
      cw.appendln(methCode_cw.toString());
    }

    public void visitField(final Field field) throws Exception {
    }

    public void visitMethod(final Method method) throws Exception {
      methCode_cw.append(buildMethodDefinition(method, false, "METHOD(input,"
          + method.getName().toString() + ")"));
      methCode_cw.appendln("{");

      for (final Parameter p : ((ParameterContainer) method).getParameters()) {
        final Type type = getContainedType(p);

        if ((type instanceof PrimitiveType)
            && (getPrimitiveType((PrimitiveType) type) == PrimitiveTypeEnum.STRING)) {
          /* char string */
          methCode_cw.appendln("if(" + p.getName() + "){");
          methCode_cw.appendln("int l=0; while (" + p.getName().toString()
              + "[l++]);");
          methCode_cw.appendln("CALL(REQUIRED.trigger, alloc, (unsigned int)"
              + p.getName() + ", l);");
          methCode_cw.appendln("}");
        }
        if (type instanceof PointerOf) {
          /* pointer */
          methCode_cw.appendln("if(" + p.getName() + "){");
          methCode_cw.appendln("CALL(REQUIRED.trigger, alloc, (unsigned int)"
              + p.getName() + ", sizeof(*" + p.getName() + "));");
          methCode_cw.appendln("}");
        }
        if (type instanceof ArrayOf) {
          /* array */
          methCode_cw.appendln("if(" + p.getName() + "){");
          methCode_cw.appendln("CALL(REQUIRED.trigger, alloc, (unsigned int)"
              + p.getName() + ", sizeof("
              + buildSizeableArray(type, ParseArrayMode.TYPE)
              + buildSizeableArray(type, ParseArrayMode.SIZEOF) + "));");
          methCode_cw.appendln("}");
        }
      }

      methCode_cw.append("CALL(REQUIRED.output, " + method.getName());
      for (final Parameter p : ((ParameterContainer) method).getParameters()) {
        methCode_cw.append(", " + p.getName());
      }
      methCode_cw.appendln(");");

      methCode_cw.appendln("}");

    }
  }
}
