/***
 * Think ADL Parser
 * Copyright (C) 2005 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: think@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.rpc.timer;

import static org.objectweb.fractal.cecilia.adl.idl.util.CUtil.buildMethodDefinition;
import static org.objectweb.fractal.cecilia.adl.idl.util.Util.getPrimitiveType;

import org.objectweb.fractal.cecilia.adl.file.CodeWriter;
import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.InterfaceDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.Method;
import org.objectweb.fractal.cecilia.adl.idl.ast.Parameter;
import org.objectweb.fractal.cecilia.adl.idl.ast.ParameterContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.PrimitiveType;
import org.objectweb.fractal.cecilia.adl.idl.ast.Type;
import org.objectweb.fractal.cecilia.adl.idl.ast.PrimitiveType.PrimitiveTypeEnum;
import org.objectweb.fractal.cecilia.adl.idl.remote.VarNameValidator;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.cecilia.adl.idl.util.Util;
import org.objectweb.fractal.cecilia.adl.interfaces.AbstractInterfaceAdapterImplementationVisitor;

/**
 * Generates the implementation of a XStream skeleton component.
 */
public class ImplementationVisitor
    extends
      AbstractInterfaceAdapterImplementationVisitor {

  @Override
  protected AbstractInterfaceAdapterImplmentationTask createImplementationTask(
      final InterfaceDefinition itf) {
    return new ImplementationTask(itf);
  }

  /** 
   *
   */
  public static class ImplementationTask
      extends
        AbstractInterfaceAdapterImplmentationTask {

    private final String     x       = new String(); // File
    // timer

    protected CodeWriter     methenum_cw;
    protected CodeWriter     methcode_cw;

    private VarNameValidator localVars;
    private String           ItfName = new String();
    private String           result;

    /**
     * @param itf the {@link InterfaceDefinition} that is processed by this
     *            task.
     */
    public ImplementationTask(final InterfaceDefinition itf) {
      super(itf);
    }

    public void enterInterface(final InterfaceDefinition itf) throws Exception {
      methenum_cw = new CodeWriter();
      methcode_cw = new CodeWriter();
      ItfName = (itf.getName().replace('.', '_'));
      methenum_cw.append("typedef enum {");
    }

    public void leaveInterface(final InterfaceDefinition itf) throws Exception {

      methenum_cw.appendln("} " + ItfName.toString() + "_meths_t;");

      cw = new CodeWriter();
      cw.endl();

      cw.append("#include <stdio.h>").endl();
      cw.appendln("DECLARE_DATA {};");
      cw.appendln("#include <cecilia.h>");
      cw.endl();

      cw.appendln(methenum_cw.toString()).endl();
      cw.appendln(methcode_cw.toString()).endl();

      cw.appendln(x);
    }

    public void visitField(final Field field) throws Exception {
    }

    // Memo: Skeleton
    public void visitMethod(final Method method) throws Exception {
      final Type returnType = Util.getContainedType((method));
      final boolean isVoidMethod = (returnType instanceof PrimitiveType)
          && getPrimitiveType((PrimitiveType) returnType) == PrimitiveTypeEnum.VOID;

      methenum_cw.append(method.getName() + "_ID,");

      methcode_cw.append(buildMethodDefinition(method, false, "METHOD(input,"
          + method.getName().toString() + ")"));
      methcode_cw.appendln("{");

      if (!isVoidMethod) {
        result = localVars.validateName("result");
        // declare result variable.
        methcode_cw.append(CUtil.buildReturnType(returnType)).append(" ")
            .append(result).append(";").endl();
      }

      // prints interface/method name
      /*
       * methcode_cw.appendln("printf(\"before " + ItfName.toString() + "::" +
       * method.getName().toString() + "\\n\");");
       */

      // trigger reporter BEGIN(methodeid)
      methcode_cw.appendln("CALL(REQUIRED.trigger,begin,(void*)"
          + method.getName() + "_ID" + ");");

      // call application method
      if (!isVoidMethod) methcode_cw.append(result).append(" = ");
      methcode_cw.append("CALL(REQUIRED.output," + method.getName().toString());
      for (final Parameter p : ((ParameterContainer) method).getParameters()) {
        methcode_cw.append("," + "\n");
        methcode_cw.append("   " + p.getName());
      }
      methcode_cw.append(");" + "\n");

      // trigger report END
      methcode_cw.appendln("CALL(REQUIRED.trigger,end);");

      // prints interface/method name
      /*
       * methcode_cw.appendln("printf(\"after " + ItfName.toString() + "::" +
       * method.getName().toString() + "\\n\");");
       */

      // returns result
      if (!isVoidMethod)
        methcode_cw.append("return ").append(result).append(";");

      methcode_cw.appendln("}");
    }
  }
}